<?php
/**
 * Internships block
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/copyleft/gpl.html.
 *
 */

/**
 * Internships block class definition
 *
 * @author     Pablo Pagnone
 * @package    block
 * @subpackage internships
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @copyright  Copyright (c) 2012 CareerArc Group LLC.
 **/

defined('MOODLE_INTERNAL') || die();

require_once($CFG->dirroot.'/blocks/moodleblock.class.php');

class block_internships extends block_base {

    private $config_block = null;

    const CACHE_MINUTES = 10;

    const DEFAULT_URL_INTERNSHIPS = 'http://www.internships.com';
    const DEFAULT_BASE_URL        = 'https://secure.internships.com/moodle';
    const DEFAULT_KEY             = 'moodlerooms';
    const DEFAULT_SECRET          = '4F8B620DF5C8401a85B41F1ECB096741';

    public function init() {
        $this->title = get_string('pluginname', __CLASS__);
        $this->config_block = get_config(__CLASS__);
    }

    public function applicable_formats() {
        return array('all' => false,
                     'site' => true,
                     'course-view' => true,
                     'my' => true);
    }

    public function has_config() {
        return true;
    }

    public function get_content() {
        global $COURSE, $PAGE;

        if ($this->content !== null) {
            return $this->content;
        }

        $this->content = new stdClass();
        $this->content->text = '';
        $this->content->footer = '';

        if (!has_capability('block/internships:connect', $this->context)) {
            return $this->content;
        }

        if (empty($this->config_block->key) or empty($this->config_block->secret)) {
            //  Settings are not configured correctly.
            if (array_key_exists($USER->id, get_admins())) {
                // If it is admin , show the message.
                $surl = new moodle_url('/admin/settings.php',
                                       array('section' => 'blocksettinginternships'));
                $this->content->text = html_writer::link($surl->out(false),
                                       get_string('error_setting_not_save', __CLASS__));
            }
            return $this->content;
        }

        // Get stats.
        $stats = $this->get_stats();
        if (is_string($stats)) {
            // Error to get stats.
            $err  = html_writer::tag('p'  , get_string('conectivity_error', __CLASS__));
            $err .= html_writer::tag('div', $stats, array('class' => 'errorbox'));
            $this->content->text = $err;
            return $this->content;
        }

        // Get renderer for show links in block.
        $render = $PAGE->get_renderer(__CLASS__);

        require_once(__DIR__.'/form/search.php');

        //  Search form with some validation.
        $form = new block_internships_form_search('#', null, 'post');
        if ($dataform = $form->get_data()) {
            if ($form->is_validated()) {
                $url_action = new moodle_url('/blocks/internships/view.php',
                                             array('what' => trim($dataform->what),
                                                   'where' => trim($dataform->where),
                                                   'courseid' => $COURSE->id ));
                redirect($url_action);
            }
        }
        // End form.

        $this->content->text = $render->block_content($stats->total_internship,
                               $stats->total_companies,
                               $form);

        return $this->content;
    }

    /**
     * Get stats from webservice
     * If webservice fail , return string with error
     * Else return object with values
     * This method implement cache to get stats from webservice ever 10 minutes
     *
     * @return string|stdClass
     */
    private function get_stats() {
        $result = new stdClass();
        // Get stats for cache or webservice.
        if (isset($this->config_block->cache_last) && ($this->config_block->cache_last + MINSECS * self::CACHE_MINUTES) > time()) {

            // Use cache.
            $result->total_internship = $this->config_block->total_internship;
            $result->total_companies = $this->config_block->total_companies;

        } else {
            require_once(__DIR__.'/locallib.php');
            // Get Statistics from webservice and save in cache.
            $client = block_internships_wsclient::instance($this->config_block);
            if ($client === null) {
                return '';
            }
            if (!$client->stats()) {
                // Error to get stats - return string.
                return $client->get_fullstatus();
            }

            $data = $client->get_last_result();
            $result->total_internship = $data['ActiveInternshipPositions'];
            $result->total_companies = $data['ActiveCompanies'];

            // Save in cache.
            set_config('cache_last', time(), 'block_internships');
            set_config('total_internship', $result->total_internship, 'block_internships');
            set_config('total_companies', $result->total_companies, 'block_internships');

        }

        return $result;
    }

    public function user_can_addto($page) {
        // User only can to add if has capability 'block/internships:addblock'.
        return has_capability('block/internships:addblock', $page->context);
    }
}