<?php
/**
 * Internships block
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/copyleft/gpl.html.
 *
 */

/**
 * Web service client infrastructure
 *
 * @package    block
 * @subpackage internships
 * @author     Darko Miletic
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 * @copyright  Copyright (c) 2012 CareerArc Group LLC.
 */

defined('MOODLE_INTERNAL') || die();

class block_internships_wsclient_base_exception extends moodle_exception {
    public function __construct($errorcode, $a=null, $debuginfo=null) {
        parent::__construct($errorcode, 'block_internships', '', $a, $debuginfo);
    }
}

class block_internships_wsclient_invalidparam_exception extends block_internships_wsclient_base_exception {
    public function __construct() {
        parent::__construct('invalidurl');
    }
}

class block_internships_wsclient_runtime_exception extends block_internships_wsclient_base_exception {
    public function __construct() {
        parent::__construct('runtime_error');
    }
}

class block_internships_wsclient {
    /**
     * @var Zend_Http_Response
     */
    protected $last_response = null;
    /**
     * @var string
     */
    protected $consumer_key = null;
    /**
     * @var string
     */
    protected $consumer_secret = null;

    protected $result_data = null;

    protected $result_type = null;

    /**
     * @var string
     */
    protected $status = null;
    /**
     * @var string
     */
    protected $description = null;
    /**
     * @var string
     */
    protected $redirect_url   = null;
    protected $register_url   = 'https://secure.internships.com/moodle/register';
    protected $sign_in_url    = 'https://secure.internships.com/moodle/sign-in';
    protected $stats_url      = 'https://secure.internships.com/moodle/stats';

    /**
     *
     * @param stdClass $config
     * @return block_internships_wsclient
     */
    public static function instance($config=null) {
        $client = null;
        if ($config === null) {
            $config = get_config('block_internships');
        }

        if (!isset($config->key) or !isset($config->secret)) {
            return $client;
        }

        $client = new block_internships_wsclient($config->key, $config->secret);
        if (!empty($config->internships_api_url)) {
            $client->set_register_endpoint($config->internships_api_url.'/register');
            $client->set_sign_in_endpoint($config->internships_api_url.'/sign-in');
            $client->set_stats_endpoint($config->internships_api_url.'/stats');
        }

        return $client;
    }

    /**
     *
     * @param string $key
     * @param string $secret
     */
    public function __construct($key, $secret) {
        $this->consumer_key = $key;
        $this->consumer_secret = $secret;
    }

    /**
     * Sets new endpoint for the register API
     * @param string $url
     */
    public function set_register_endpoint($url) {
        $this->register_url = $url;
    }

    /**
     * Sets new endpoint for the sign-in API
     * @param string $url
     */
    public function set_sign_in_endpoint($url) {
        $this->sign_in_url = $url;
    }

    /**
     * Sets new endpoint for the stats API
     * @param string $url
     */
    public function set_stats_endpoint($url) {
        $this->stats_url = $url;
    }

    /**
     * Internal method for calling REST endpoint
     * @param string $url
     * @param array $params
     * @return boolean
     * @throws block_internships_wsclient_invalidparam_exception
     */
    protected function call_wsmethod($url, array $params = array()) {
        $result = false;
        if (empty($url)) {
            throw new block_internships_wsclient_invalidparam_exception();
        }

        // Setup Zend includes.
        mr_bootstrap::zend();

        require_once('Zend/Json.php');
        require_once('Zend/Oauth/Consumer.php');
        require_once('Zend/Oauth/Client.php');

        $aconfig = array(
                       'requestScheme'   => Zend_Oauth::REQUEST_SCHEME_HEADER,
                       'requestMethod'   => Zend_Oauth::POST,
                       'signatureMethod' => 'HMAC-SHA1',
                       'consumerKey'     => $this->consumer_key,
                       'consumerSecret'  => $this->consumer_secret
                   );

        $tacc = new Zend_Oauth_Token_Access();
        $client = $tacc->getHttpClient($aconfig);
        $client->setUri($url);
        $client->setMethod(Zend_Http_Client::POST);
        foreach ($params as $name => $value) {
            $client->setParameterPost($name, $value);
        }
        try {
            $response = $client->request();
            $this->last_response = $response;
            $result_body = $response->getBody();
            $result = $response->isSuccessful();

            // Get content type.
            $this->result_type = $response->getHeader('Content-Type');
            // Is this Json encoded data?
            if (stripos($this->result_type, 'application/json') !== false) {
                $this->result_data = Zend_Json::decode($result_body);
            } else {
                $this->result_data = $result_body;
            }

            // By default set the status to the HTTP response status.
            $this->status      = (string)$response->getStatus();
            $this->description = $response->getMessage();
        } catch (Exception $e) {
            $result = false;
            $this->status      = (string)$e->getCode();
            $this->description = $e->getMessage();
        }

        return $result;
    }

    /**
     * Obtains the content result of the last method call
     * It can be either array, text or HTML
     * @return mixed
     */
    public function get_last_result() {
        return $this->result_data;
    }

    /**
     * Returns content type of the last returned result
     * @return string
     */
    public function get_last_content_type() {
        return $this->result_type;
    }

    /**
     * Returns raw HTTP response object that can be used for advanced debugging
     * @return Zend_Http_Response
     */
    public function get_last_response() {
        return $this->last_response;
    }

    public function get_status() {
        return $this->status;
    }

    public function get_description() {
        return $this->description;
    }

    public function get_fullstatus() {
        $result = sprintf("Status: %s\nMessage: %s", $this->status, $this->description);
        return $result;
    }

    public function get_redirect_url() {
        return $this->redirect_url;
    }

    /**
     * This method registers user or checks if user is already registered
     * $params array must contain these elements (all are required)
     * array (
     *     'first_name' => 'users first name'
     *    ,'last_name'  => 'users last name'
     *    ,'city'       => 'users city'
     *    ,'country'    => 'users country'
     *    ,'email'      => 'users email'
     *    ,'allow_notifications' => 'true' or 'false' (as string)
     *    );
     * @param array $params
     * @return boolean
     */
    public function register(array $params) {
        $result = $this->call_wsmethod($this->register_url, $params);
        if ($result) {
            if (is_array($this->result_data)) {
                $this->status = $this->result_data['status'];
                if (isset($this->result_data['error'])) {
                    $result = false;
                    $this->description = $this->result_data['error'];
                }
                if (isset($this->result_data['message'])) {
                    $this->description = $this->result_data['message'];
                }
            }
        }
        return $result;
    }

    /**
     * This method sign's in the user
     * If we add either where or what variables than method returns
     * html page that contains internships returned as result of the search.
     * @param array $params
     * @param string $what
     * @param string $where
     * @return mixed
     */
    public function sign_in(array $params, $what = null, $where = null) {
        if ($what !== null) {
            $params['what'] = $what;
        }
        if ($where !== null) {
            $params['where'] = $where;
        }
        $result = $this->call_wsmethod($this->sign_in_url, $params);
        if ($result) {
            if (is_array($this->result_data)) {
                $result = false;
                $this->status = $this->result_data['status'];
                if ((strcasecmp($this->status, 'Registered') == 0) ||
                        (strcasecmp($this->status, 'Pending') == 0) ) {
                    $result = true;
                }
                if (isset($this->result_data['error'])) {
                    $this->description = $this->result_data['error'];
                } else if (isset($this->result_data['message'])) {
                    $this->description = $this->result_data['message'];
                }
                if (isset($this->result_data['redirect'])) {
                    $this->redirect_url = $this->result_data['redirect'];
                }
            }
        }
        return $result;
    }

    /**
     *
     * @return boolean
     */
    public function stats() {
        $result = $this->call_wsmethod($this->stats_url);
        if ($result) {
            if (is_array($this->result_data)) {
                if (isset($this->result_data['status'])) {
                    $result = false;
                    //  We got some error.
                    $this->status = $this->result_data['status'];
                    if (isset($this->result_data['error'])) {
                        $this->description = $this->result_data['error'];
                    }
                    if (isset($this->result_data['message'])) {
                        $this->description = $this->result_data['message'];
                    }
                }
            }
        }
        return $result;
    }

}